/*
* Copyright (C) 2008  Valery V. Inozemtsev <shrek@altlinux.org>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include "SDL/SDL.h"
#include "SDL/SDL_mixer.h"

extern const char *__progname;

static struct option const longopts[] = {
        {"volume", required_argument, 0, 'v'},
        {"driver", required_argument, 0, 'd'},
        {"rate", required_argument, 0, 'r'},
        {"help", no_argument, 0, 'h'},
        {0, no_argument, 0, 0}
};

static void __attribute__ ((__noreturn__)) usage (FILE * stream, int status)
{
	fprintf (stream, "\
%s - SDL audio player.\n\
Usage: %s [-v | --volume 0..100] [-d | --driver alsa|pulse] [-r | --rate 8000|11025|16000|22050|24000|32000|44100|48000] input file\n\
       %s [-h | --help]\n\
\nReport bugs to http://bugs.altlinux.ru/\n", __progname, __progname, __progname);
        exit (status);
}

int main(int argc, char *argv[])
{
	int c;
	int audio_rate = 22050;
	Uint16 audio_format = AUDIO_S16;
	int audio_channels = 2;
	int audio_buffers = 4096;
	int volume = 100;
	char *driver = NULL;
	Mix_Music *music = NULL;
	char *filename;

	while ((c = getopt_long (argc, argv, "v:d:r:h", longopts, 0)) != -1)
		switch (c) {
			case 'v':
				volume = atoi(optarg);
				break;
			case 'd':
				driver = optarg;
				if ((strcmp(driver, "alsa") != 0) && (strcmp(driver, "pulse") != 0))
					usage(stderr, EXIT_FAILURE);
				break;
			case 'r':
				if ((strcmp(optarg, "8000") != 0) && (strcmp(optarg, "11025") != 0) &&
					(strcmp(optarg, "16000") != 0) && (strcmp(optarg, "22050") != 0) &&
					(strcmp(optarg, "24000") != 0) && (strcmp(optarg, "32000") != 0) &&
					(strcmp(optarg, "44100") != 0) && (strcmp(optarg, "48000") != 0))
						usage(stderr, EXIT_FAILURE);
				audio_rate = atoi(optarg);
				break;
			case 'h':
				usage(stdout, EXIT_SUCCESS);
				break;
			default:
				usage(stderr, EXIT_FAILURE);
		}

	filename = argv[optind++];
	if (!filename || !*filename)
		usage (stderr, EXIT_FAILURE);

	if (driver != NULL)
		setenv("SDL_AUDIODRIVER", driver, 1);

	if (SDL_Init(SDL_INIT_AUDIO) < 0) {
		fprintf(stderr, "Couldn't initialize SDL: %s\n", SDL_GetError());
		return 1;
	}

	if (Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers)) {
		fprintf(stderr, "Couldn't open audio: %s\n", SDL_GetError());
		return 1;
	}

	music = Mix_LoadMUS(filename);

	if (music == NULL) {
		fprintf(stderr, "Couldn't load %s: %s\n", filename, SDL_GetError());
		return 1;
	}

	if (volume > 100)
		volume = 100;
	if (volume < 0)
		volume = 0;

	Mix_VolumeMusic((MIX_MAX_VOLUME / 100) * volume);
	if (Mix_PlayMusic(music, 0) == -1)
		fprintf(stderr, "Mix_PlayMusic: %s\n", Mix_GetError());

	while (Mix_PlayingMusic())
		SDL_Delay(100);

	Mix_HaltMusic();
	Mix_FreeMusic(music);
	music = NULL;

	Mix_CloseAudio();
	SDL_Quit();

	return 0;
}
